package de.fraunhofer.iem.dataprovider.toolRun.service

import de.fraunhofer.iem.dataprovider.gitlab.enumeration.PlatformEnum
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.repository.service.RepositoryService
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.sarif.json.SarifJson
import de.fraunhofer.iem.dataprovider.taskManager.tasks.tools.sarif.json.asDbObject
import de.fraunhofer.iem.dataprovider.tool.service.RuleService
import de.fraunhofer.iem.dataprovider.tool.service.ToolService
import de.fraunhofer.iem.dataprovider.toolRun.entity.ToolRunEntity
import de.fraunhofer.iem.dataprovider.toolRun.repository.ToolRunRepository
import org.springframework.stereotype.Service
import java.util.*

@Service
class ToolRunService(
    private val repositoryService: RepositoryService,
    private val toolService: ToolService,
    private val ruleService: RuleService,
    private val toolRunRepository: ToolRunRepository,
) {

    private val logger = getLogger(javaClass)

    fun getLatestToolRunsForRepo(repoId: Long, platform: PlatformEnum): List<ToolRunEntity> {
        val repo = repositoryService.findRepoByRepoIdAndPlatform(repoId, platform)
        if (repo !== null) {
            return getLatestToolResultsForRepo(repo)
        } else {
            logger.warn("Requested repository doesn't exist.")
        }
        return emptyList()
    }


    private fun getLatestToolResultsForRepo(repo: RepositoryEntity): List<ToolRunEntity> {
        return repo.toolEntities.map { tool ->
            toolRunRepository.findFirstByTool_IdAndRepository_IdOrderByCreatedAtDesc(tool.id!!, repo.id!!)
        }
    }

    // TODO: this func should be a suspend fun. We can achieve this by
    //  wrapping the db return types in flux, mono, or futures
    fun createToolRunFromSarif(sarifJson: SarifJson, repoId: UUID) {

        val repo = repositoryService.findRepoByID(repoId)

        sarifJson.runs.forEach { run ->

            val tool = toolService.findOrCreateTool(run.tool)
            val rules = ruleService.getRulesByToolId(tool.id).toMutableList()

            ruleService.createNewRulesAndSaveToTool(run.tool.driver.rules, rules, tool)


            val toolResults = run.results.map { sarifResult ->
                val tr = sarifResult.asDbObject()
                // link rules to results
                rules.filter { it.sarifRuleId == sarifResult.ruleId }
                    .forEach { it.addResultLocation(tr) }
                tr
            }


            val toolRun = run.asDbObject()
            toolRun.tool = tool
            toolRun.addRules(rules)
            toolRun.addFindings(toolResults)

            val result = toolRunRepository.save(toolRun)
            toolService.save(tool)

            if (repo != null) {
                if (!repo.toolEntities.contains(tool)) {
                    repo.toolEntities.add(tool)
                }
                repo.addToolResult(result)
                repositoryService.save(repo)
            }
        }
    }
}
